-- Shaman.lua
-- October 2016

local addon, ns = ...
local Hekili = _G[ addon ]

local class = ns.class
local state = ns.state

local addHook = ns.addHook

local addAbility = ns.addAbility
local modifyAbility = ns.modifyAbility
local addHandler = ns.addHandler

local addAura = ns.addAura
local modifyAura = ns.modifyAura

local addGearSet = ns.addGearSet
local addGlyph = ns.addGlyph
local addMetaFunction = ns.addMetaFunction
local addTalent = ns.addTalent
local addPerk = ns.addPerk
local addResource = ns.addResource
local addStance = ns.addStance

local registerCustomVariable = ns.registerCustomVariable
local registerInterrupt = ns.registerInterrupt

local removeResource = ns.removeResource

local setClass = ns.setClass
local setPotion = ns.setPotion
local setRole = ns.setRole

local RegisterEvent = ns.RegisterEvent
local storeDefault = ns.storeDefault


-- This table gets loaded only if there's a supported class/specialization.
if (select(2, UnitClass('player')) == 'SHAMAN') then

    ns.initializeClassModule = function ()

        setClass( 'SHAMAN' )

        -- addResource( SPELL_POWER_HEALTH )
        addResource( 'mana', true )
        addResource( 'maelstrom' )

        addTalent( 'windsong', 201898 )
        addTalent( 'hot_hand', 201900 )
        addTalent( 'boulderfist', 201897 )

        addTalent( 'rainfall', 215864 )
        addTalent( 'feral_lunge', 196884 )
        addTalent( 'wind_rush_totem', 192077 )

        addTalent( 'lightning_surge_totem', 192058 )
        addTalent( 'earthgrab_totem', 51485 )
        addTalent( 'voodoo_totem', 196932 )

        addTalent( 'lightning_shield', 192106 )
        addTalent( 'ancestral_swiftness', 192087 )
        addTalent( 'hailstorm', 210853 )

        addTalent( 'tempest', 192234 )
        addTalent( 'overcharge', 210727 )
        addTalent( 'empowered_stormlash', 210731 )

        addTalent( 'crashing_storm', 192246 )
        addTalent( 'fury_of_air', 197211 )
        addTalent( 'sundering', 197214 )

        addTalent( 'ascendance', 114051 )
        addTalent( 'landslide', 197992 )
        addTalent( 'earthen_spike', 188089 )


        -- Player Buffs.
        addAura( 'ascendance', 114051 )
        addAura( 'boulderfist', 218825 )
        addAura( 'crash_lightning', 187874 )
        addAura( 'doom_winds', 204945 )
        addAura( 'flametongue', 194084 )
        addAura( 'frostbrand', 196834 )
        addAura( 'fury_of_air', 197211 )
        addAura( 'hot_hand', 215785 )
        addAura( 'landslide', 202004 )
        addAura( 'lightning_shield', 192109 )
        addAura( 'rainfall', 215864 )
        addAura( 'stormbringer', 201845 )
        addAura( 'windsong', 201898 )


        -- Fake Buffs.
        registerCustomVariable( 'last_feral_spirit', 0 )
        registerCustomVariable( 'last_crash_lightning', 0 )
        registerCustomVariable( 'last_rainfall', 0 )

        RegisterEvent( "UNIT_SPELLCAST_SUCCEEDED", function( _, unit, spell, _, spellID )

            if unit ~= 'player' then return end

            if spell == class.abilities[ 'feral_spirit' ].name then
                state.last_feral_spirit = GetTime()
            
            elseif spell == class.abilities[ 'crash_lightning' ].name then
                state.last_crash_lightning = GetTime()

            elseif spell == class.abilities[ 'rainfall' ].name then
                state.last_rainfall = GetTime()

            end

        end )

        addAura( 'feral_spirit', -10, 'name', 'Feral Spirit', 'duration', 15, 'feign', function ()
            local up = last_feral_spirit
            buff.feral_spirit.name = 'Feral Spirit'
            buff.feral_spirit.count = up and 1 or 0
            buff.feral_spirit.expires = up and last_feral_spirit + 15 or 0
            buff.feral_spirit.applied = up and last_feral_spirit or 0
            buff.feral_spirit.caster = 'player'
        end )

        addAura( 'alpha_wolf', -11, 'name', 'Alpha Wolf', 'duration', 8, 'feign', function ()
            local time_since_cl = now + offset - last_crash_lightning        
            local up = buff.feral_spirit.up and last_crash_lightning > buff.feral_spirit.applied
            buff.alpha_wolf.name = 'Alpha Wolf'
            buff.alpha_wolf.count = up and 1 or 0
            buff.alpha_wolf.expires = up and last_crash_lightning + 8 or 0
            buff.alpha_wolf.applied = up and last_crash_lightning or 0
            buff.alpha_wolf.caster = 'player'
        end )


        state.feral_spirit = setmetatable( {}, {
            __index = function( t, k )
                if k == 'cast_time' then
                    t.cast_time = state.last_feral_spirit and state.last_feral_spirit or 0
                    return t[k]
                elseif k == 'active' then
                    return state.query_time < t.cast_time + 15 or false
                elseif k == 'remains' then
                    return max( 0, t.cast_time + 15 - state.query_time )
                end

                return false
            end
        } )

        ns.addHook( 'reset_postcast', function( x )
            state.feral_spirit.cast_time = nil 
            return x
        end )

        -- Pick an instant cast ability for checking the GCD.
        -- setGCD( 'global_cooldown' )

        -- Gear Sets
        addGearSet( 'tier19', 138341, 138343, 138345, 138346, 138348, 138372 )
        addGearSet( 'class', 139698, 139699, 139700, 139701, 139702, 139703, 139704, 139705 )
        addGearSet( 'doomhammer', 128819 )


        addHook( 'specializationChanged', function ()
            setPotion( 'old_war' )
            setRole( 'attack' )
        end )


        ns.addToggle( 'doom_winds', true, 'Doom Winds', 'Set a keybinding to toggle Doom Winds on/off in your priority lists.' )

        ns.addSetting( 'prioritize_buffs', true, {
            name = "Prioritize Buffs",
            type = "toggle",
            desc = "If |cFF00FF00true|r, the addon will use a modified priority list that recommends maintaining your Frostbrand and Flametongue buffs before hitting Stormstrike.  " ..
                "This deviates from the default SimulationCraft action list as of October 2016, but tends to show higher performance and maintains the good habit of keeping your " ..
                "weapon buffs active.",
            width = "full"
        } )

        ns.addSetting( 'doom_winds_cooldown', true, {
            name = "Doom Winds: Cooldown Override",
            type = "toggle",
            desc = "If |cFF00FF00true|r, when your Cooldown toggle is |cFF00FF00ON|r then the toggle for Doom Winds will be overriden and Doom Winds will be shown regardless of your Doom Winds toggle.",
            width = "full"
        } )

        ns.addSetting( 'boulderfist_maelstrom', 100, {
            name = "Maelstrom: Boulderfist",
            type = "range",
            desc = "Set a |cFFFF0000maximum|r amount of Maelstrom for the Boulderfist ability (if talented) in the default action lists.  This is useful if you are concerned with wasting Maelstrom by using Boulderfist when some/all of the Maelstrom it would generate would be wasted.\n\n" ..
                "The addon default is 100, but is ignored if your Boulderfist/Landslide buffs will fall off within a global cooldown.  SimulationCraft and Wordup both recommend Boulderfist without regard for Maelstrom overcapping.\r\n" ..
                "You can incorporate this into your custom APLs using the |cFFFFD100settings.boulderfist_maelstrom|r syntax.",
            min = 0,
            max = 150,
            step = 1,
            width = 'full'
        } )

        ns.addSetting( 'crash_lightning_maelstrom', 0, {
            name = "Maelstrom: Crash Lightning",
            type = "range",
            desc = "Set a |cFFFF0000minimum|r amount of Maelstrom required to recommend Crash Lightning in the default action lists.  This is useful if you are concerned with maintaining a minimum Maelstrom pool to use on Stormbringer procs, etc.\n\n" ..
                "The addon default is 0 (but obeys the cost of Crash Lightning).  Wordup's Enhancement Shaman guide on Wowhead recommends Crash Lightning in Single Target if you have > 80 Maelstrom.\n\n" ..
                "You can incorporate this into your custom APLs using the |cFFFFD100settings.crash_lightning_maelstrom|r syntax.",
            min = 0,
            max = 150,
            step = 1,
            width = 'full'
        } )

        ns.addSetting( 'lava_lash_maelstrom', 90, {
            name = "Maelstrom: Lava Lash",
            type = "range",
            desc = "Set a |cFFFF0000minimum|r amount of Maelstrom required to cast Lava Lash in the default action lists.  This is ignored if Lava Lash would currently be free.\n\n" ..
                "The addon default, Wordup's Wowhead guide, and SimulationCraft all recommend using Lava Lash at/above 90 Maelstrom by default.\n\n" .. 
                "You can incorporate this into your custom APLs using the |cFFFFD100settings.lava_lash_maelstrom|r syntax.",
            min = 0,
            max = 150,
            step = 1,
            width = 'full'
        } )

        ns.addSetting( 'sundering_maelstrom', 0, {
            name = "Maelstrom: Sundering",
            type = "range",
            desc = "Set a |cFFFF0000minimum|r amount of Maelstrom required to recommend Sundering (if talented) in the default action lists.  This is useful if you are concerned with maintaining a minimum Maelstrom pool to use on Stormbringer procs, etc.\n\n" ..
                "The addon default is 0 (but obeys the cost of Sundering).  Wordup's Enhancement Shaman guide on Wowhead recommends Sundering if you have > 110 Maelstrom.\n\n" ..
                "You can incorporate this into your custom APLs using the |cFFFFD100settings.sundering_maelstrom|r syntax.",
            min = 0,
            max = 150,
            step = 1,
            width = 'full'
        }   )


        addAbility( 'ascendance', {
            id = 114051,
            spend = 0,
            spend_type = 'mana',
            cast = 0,
            gcdType = 'off',
            known = function () return talent.ascendance.enabled end,
            cooldown = 180,
            toggle = 'cooldowns'
        } )

        addHandler( 'ascendance', function ()
            applyBuff( 'ascendance', 15 )
            setCooldown( 'stormstrike', 0 )
        end )


        addAbility( 'bloodlust', {
            id = 2825,
            spend = 0.215,
            cast = 0,
            gcdType = 'off',
            cooldown = 300,
            toggle = 'cooldowns'
        } )


        addAbility( 'boulderfist', {
            id = 201897,
            spend = -25,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            known = function () return talent.boulderfist.enabled end,
            cooldown = 6,
            charges = 2,
            recharge = 6
        } )

        addHandler( 'boulderfist', function ()
            applyBuff( 'boulderfist', 10 )
            if talent.landslide.enabled then
                applyBuff( 'landslide', 10 )
            end
        end )

        modifyAbility( 'boulderfist', 'cooldown', function( x )
            return x * haste
        end )

        modifyAbility( 'boulderfist', 'recharge', function( x )
            return x * haste
        end )


        addAbility( 'crash_lightning', {
            id = 187874,
            spend = 20,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 6
        } )

        addHandler( 'crash_lightning', function ()
            if active_enemies > 2 then
                applyBuff( 'crash_lightning', 10 )
            end

            -- LegionFix:  Artifact Check
            if feral_spirit.active then
                applyBuff( 'alpha_wolf', min( 8, buff.feral_spirit.remains ) )
            end
        end )

        modifyAbility( 'crash_lightning', 'cooldown', function( x )
            return x * haste
        end )


        addAbility( 'doom_winds', {
            id = 204945,
            spend = 0,
            spend_type = 'mana',
            cast = 0,
            gcdType = 'off',
            cooldown = 60,
            known = function() return equipped.doomhammer and ( toggle.doom_winds or ( toggle.cooldowns and settings.doom_winds_cooldown ) ) end,
            -- toggle = 'cooldowns'
        } )

        addHandler( 'doom_winds', function ()
            applyBuff( 'doom_winds', 6 )
        end )


        addAbility( 'earthen_spike', {
            id = 188089,
            spend = 30,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 20,
            known = function() return talent.earthen_spike.enabled end
        } )

        addHandler( 'earthen_spike', function ()
            applyDebuff( 'target', 'earthen_spike', 10 )
        end )


        addAbility( 'feral_spirit', {
            id = 51533,
            spend = 0,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 120,
            toggle = 'cooldowns'
        } )

        addHandler( 'feral_spirit', function ()
            summonPet( 'feral_spirit', 15 )
            feral_spirit.cast_time = state.now + state.offset
        end )


        addAbility( 'flametongue', {
            id = 193796,
            spend = 0,
            spend_type = 'mana',
            cast = 0,
            gcdType = 'spell',
            cooldown = 12
        } )

        modifyAbility( 'flametongue', 'cooldown', function( x )
            return x * haste
        end )

        addHandler( 'flametongue', function ()
            applyBuff( 'flametongue', 16 + min( 4.8, buff.flametongue.remains ) )
        end )


        addAbility( 'frostbrand', {
            id = 196834,
            spend = 20,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 0,
        } )

        addHandler( 'frostbrand', function ()
            applyBuff( 'frostbrand', 16 + min( 4.8, buff.frostbrand.remains ) )
        end )


        addAbility( 'fury_of_air', {
            id = 197211,
            spend = 5,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 0,
            known = function() return talent.fury_of_air.enabled end
        } )

        modifyAbility( 'fury_of_air', 'gcdType', function( x )
            if buff.fury_of_air.up then return 'off' end
            return x
        end )

        addHandler( 'fury_of_air', function ()
            if buff.fury_of_air.up then
                removeBuff( 'fury_of_air' )
            else
                applyBuff( 'fury_of_air', 3600 )
            end
        end )


        addAbility( 'healing_surge', {
            id = 188070,
            spend = 0,
            spend_type = 'maelstrom',
            cast = 2.0,
            gcdType = 'spell',
            cooldown = 0,
            known = function() return mana.current / mana.max > 0.22 end
        } )

        modifyAbility( 'healing_surge', 'cast', function( x )
            if maelstrom.current >= 20 then
                return 0
            end
            return x
        end )

        modifyAbility( 'healing_surge', 'spend', function( x )
            return maelstrom.current >= 20 and 20 or x
        end )


        addAbility( 'heroism', {
            id = 32182,
            spend = 0.215,
            cast = 0,
            gcdType = 'off',
            cooldown = 300,
            toggle = 'cooldowns'
        } )

        if state.faction == 'Alliance' then
            class.abilities.bloodlust = class.abilities.heroism
        else
            class.abilities.heroism = class.abilities.bloodlust
        end

        addAbility( 'lava_lash', {
            id = 60103,
            spend = 30,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'melee',
            cooldown = 0
        } )

        modifyAbility( 'lava_lash', 'spend', function( x )
            if buff.hot_hand.up then return 0 end
            return x
        end )

        addHandler( 'lava_lash', function ()
            removeBuff( 'hot_hand' )
            if buff.rainfall.up then
                buff.rainfall.expires = max( buff.rainfall.applied + 30, buff.rainfall.expires + 3 )
            end
        end )


        addAbility( 'lightning_bolt', {
            id = 187837,
            spend = 0,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 0
        } )

        modifyAbility( 'lightning_bolt', 'spend', function( x )
            if talent.overcharge.enabled then
                return min( maelstrom.current, 45 )
            end
            return x
        end )

        modifyAbility( 'lightning_bolt', 'cooldown', function( x )
            if talent.overcharge.enabled then
                return 9 * haste
            end
            return x
        end )


        addAbility( 'lightning_shield', {
            id = 192106,
            spend = 0,
            spend_type = 'mana',
            cast = 0,
            gcdType = 'spell',
            cooldown = 0,
            known = function() return talent.lightning_shield.enabled end,
            usable = function() return buff.lightning_shield.remains < 30 end,
        } )

        addHandler( 'lightning_shield', function ()
            applyBuff( 'lightning_shield', 3600 )
        end )


        addAbility( 'rainfall', {
            id = 215864,
            spend = 0,
            spend_type = 'mana',
            cast = 0,
            gcdType = 'spell', 
            cooldown = 10,
            known = function() return talent.rainfall.enabled end
        } )

        addHandler( 'rainfall', function ()
            applyBuff( 'rainfall', 10 )
        end )


        -- LegionFix: Adjust spend value based on artifact traits.
        addAbility( 'rockbiter', {
            id = 193786,
            spend = -15,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 0,
            known = function() return not talent.boulderfist.enabled end
        } )


        addAbility( 'stormstrike', {
            id = 17364,
            spend = 40,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'melee',
            cooldown = 15,
            known = function() return not buff.ascendance.up end
        } )

        modifyAbility( 'stormstrike', 'spend', function( x )
            return buff.stormbringer.up and x / 2 or x
        end )

        modifyAbility( 'stormstrike', 'cooldown', function( x )
            return buff.stormbringer.up and 0 or x * haste
        end )

        addHandler( 'stormstrike', function ()
            if buff.rainfall.up then
                buff.rainfall.expires = min( buff.rainfall.applied + 30, buff.rainfall.expires + 3 )
            end
            removeStack( 'stormbringer' )
        end )


        addAbility( 'sundering', {
            id = 197214,
            spend = 60,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 40,
            known = function() return talent.sundering.enabled end
        } )


        addAbility( 'wind_shear', {
            id = 57994,
            spend = 0,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'off',
            cooldown = 12,
            usable = function() return toggle.interrupts and target.casting end,
            toggle = 'interrupts'
        } )

        addHandler( 'wind_shear', function ()
            interrupt()
        end )


        registerInterrupt( 'wind_shear' )


        addAbility( 'windsong', {
            id = 201898,
            spend = 0,
            spend_type = 'mana',
            cast = 0,
            gcdType = 'spell',
            cooldown = 45,
            known = function() return talent.windsong.enabled end
        } )

        addHandler( 'windsong', function ()
            applyBuff( 'windsong', 20 )
        end )


        addAbility( 'windstrike', {
            id = 115356,
            spend = 40,
            spend_type = 'maelstrom',
            cast = 0,
            gcdType = 'spell',
            cooldown = 15,
            known = function() return buff.ascendance.up end
        } )

        modifyAbility( 'windstrike', 'spend', function( x )
            return buff.stormbringer.up and x / 2 or x
        end )

        modifyAbility( 'windstrike', 'cooldown', function( x )
            return buff.stormbringer.up and 0 or x * haste
        end )

        addHandler( 'windstrike', function ()
            if buff.rainfall.up then
                buff.rainfall.expires = min( buff.rainfall.applied + 30, buff.rainfall.expires + 3 )
            end
            removeStack( 'stormbringer' )
        end )

    end

    storeDefault( 'SimC Import: default', 'actionLists', 20161016.1, [[d0JkwaGEuvQnHQQDb02Oq7trPzkKy2OmFqrhwQNtQBdLhdYoHQ9QA3I2pagfLsdJsgNIcopLQHckLgmanCuPdHQcNIsXXuY5aLIfkGLkuTyLA5qEiQk5PelduToffsturHAQkIjRW0P6IcLtl5YixhqBuiP)sbBwqBhuY3vu1xrvrtduQMhOWivui(ManAuLNHkoPI0NPORPOO7POYRfs9BsoOq8xFYLyzVz04bUG3y0fPW4laaCkgxfYNrbaWbf2az(Lzmf2az(dCjoXOwthhU1Y4YYc2aUUiqOIRF5seiVuP(to(6tUel7nJgpWfbcvC9lUY0KrGv6ecbKRRVezxSYTFz(khg08OgDzAokO2vOlPkPlXjg1A64WTwgxbbT4SUG3y0f(SYbaak8OgD)4W)KlXYEZOX3xeiuX1V4kttgbcPuSHA(uZVTEJmjhKh1mNhixihgWNjmHPxy0SwGZ0YYMlr2fRC7x2mLAWaQ9ltZrb1UcDjvjDjoXOwthhU1Y4kiOfN1f8gJUmJqivPXUFCoFYLyzVz04bUG3y0LzSsHbaG8zLd9Li7IvU9ldLcZW8vo0xMMJcQDf6sQs6sCIrTMooCRLXvqqloRlceQ46x2addbdzng5Q0eibIiSUs9SZzKFB5dVzu6G86IPDfcdKYEZObmH5gyyiyisP9nQZbiIW6k1ZoNrBUFCy)tUel7nJgpWfbcvC9lUY0KrGqkfBOMp1xISlw52VesOMzqZTqLFzAokO2vOlPkPlXjg1A64WTwgxbbT4SUG3y0LOsOMbaGc3cv(9JpZp5sSS3mA8axeiuX1V0iVcBihSzC51gMxXcbPS3mAWFiHAMbn3cvoiIW6k1ZAcn4FdmmeSzC51gMxXcbrewxPggMqJlr2fRC7xGqTMNbwzYZZknVmnhfu7k0LuL0L4eJAnDC4wlJRGGwCwxWBm6cFHAnpaamkLjppR08(Xn(jxIL9MrJh4IaHkU(fxzAYiqiLInuZNA(TnKqnZGMBHkheryDL6zNjmHP3itYb9cJm4kdJIGXCCSS5seKP(s2y0Ck2WW8n6sKDXk3(fUkVu5LP5OGAxHUKQKUeNyuRPJd3AzCfe0IZ6cEJrxGTkVu59Jh8tUel7nJgpWfbcvC9lUY0KrGqkfBOMp18B7gyyiyRHOC0jebcKlmHjF4nJshS1quo6eIaPS3mAatyYiyrmySSSS5sKDXk3(LnH0ek6knVmnhfu7k0LuL0L4eJAnDC4wlJRGGwCwxWBm6sacPju0vAE)4ZWNCjw2BgnEGlceQ46xCLPjJaHuk2qnFQVezxSYTFzZuQHHqGi7xMMJcQDf6sQs6sCIrTMooCRLXvqqloRl4ngDjatPgaayubISF)4WMp5sSS3mA8axeiuX1V4nJshec1AEvAAq7kegiL9Mrd(BiVGfzGscRi9SZX5sKDXk3(fiuR5zGvM88SsZltZrb1UcDjvjDjoXOwthhU1Y4kiOfN1f8gJUWxOwZdaaJszYZZknbaqBx2C)4lRp5sSS3mA8axeiuX1VSbggcUvazdcfwAheryDLAyycn4hRjM2rkmqiGieLomMBMw83qEblYaLewr6zNJZLi7IvU9lBfq2GqHL2Vmnhfu7k0LuL0L4eJAnDC4wlJRGGwCwxWBm6safq2GqHL2VF816tUel7nJgpWfbcvC9lBGHHGBfq2GqHL2brewxPggMqdyctBH41itsBie1qEPYMn7cm4m5hRjM2rkmqiGieLomMttUxPPgCRaYgekS0UbSMyAhPW4VH8cwKbkjSI0Wyo42CjYUyLB)YwbKniuyP9ltZrb1UcDjvjDjoXOwthhU1Y4kiOfN1f8gJUeqbKniuyPDaa02Ln3p(c(NCjw2BgnEGlceQ46x2addbHqTMNbwzYZZknbrewxPggMqd(BiVGfzGscRi9SZb)sKDXk3(fiuR5zGvM88SsZltZrb1UcDjvjDjoXOwthhU1Y4kiOfN1f8gJUWxOwZdaaJszYZZknbaqBHBZ9JV48jxIL9MrJh4IaHkU(fn5ELMAqUOsHkV4BYWgyyOMFVzu6G86IPDfcdKYEZOb)BGHHGHiL23OohGicRRuddtOXLi7IvU9lHiL23OohxMMJcQDf6sQs6sCIrTMooCRLXvqqloRl4ngDjQiL23OohaaOTWwuPqLB3M7hFb7FYLyzVz04bUiqOIRFrtUxPPgKlQuOYl(MmSbggQ5FdmmemK1yKRstGeiIW6k1WWeACjYUyLB)siRXixLMaPltZrb1UcDjvjDjoXOwthhU1Y4kiOfN1f8gJUevwJrUknbsaaOTWwuPqLB3M7hFnZp5sSS3mA8axeiuX1V0qEblYaLewr6zNJd)8H3mkDqEDX0UcHbszVz04sKDXk3(L5RCODufnDzAokO2vOlPkPlXjg1A64WTwgxbbT4SUG3y0f(SYH2rv009JVm(jxIL9MrJh4IaHkU(LgYlyrgOKWksp7CC4Np8MrPdYRlM2vimqk7nJgxISlw52VODfct7OkA6Y0CuqTRqxsvsxItmQ10XHBTmUccAXzDbVXOlIRqyAhvrt3p(k4NCjw2BgnEGlceQ46x2addb1UcHTrvAsiqGCVezxSYTFz(khAhvrtxMMJcQDf6sQs6sCIrTMooCRLXvqqloRl4ngDHpRCODufnbaG2US5(XxZWNCjw2BgnEGlceQ46x2addb1UcHTrvAsiqGCVezxSYTFr7keM2rv00LP5OGAxHUKQKUeNyuRPJd3AzCfe0IZ6cEJrxexHW0oQIMaaqBx2C)4lyZNCjw2Bgn((IaHkU(f(qtUxPPgKlQuOYl(MmSbggQ53BgLoiVUyAxHWaPS3mAW)gyyiyisP9nQZbiIW6k1WWeACjYUyLB)sisP9nQZXLP5OGAxHUKQKUeNyuRPJd3AzCfe0IZ6cEJrxIksP9nQZX9Jd36tUel7nJgpWfbcvC9l8HMCVstnixuPqLx8nzydmmuZ)gyyiyiRXixLMajqeH1vQHHj04sKDXk3(LqwJrUknbsxMMJcQDf6sQs6sCIrTMooCRLXvqqloRl4ngDjQSgJCvAcKUFC4Rp5sSS3mA8axWBm6cFw5qRsZlr2fRC7xMVYHwLMxMMJcQDf6sQs6sCIrTMooCRLXvqqloR7hho8p5sSS3mA8axeiuX1V4kttgbcPuSHA(uFjYUyLB)sRHOC0jeDzAokO2vOlPkPlXjg1A64WTwgxbbT4SUG3y0LiAikhDcr3poCoFYLyzVz04bUiqOIRFHp8ck6knVezxSYTFjeiYUbvOHUqxMMJcQDf6sQs6sCIrTMooCRLXvqqloRl4ngDjQar2baqviaagPq3poCy)tUel7nJgpWfbcvC9lnKxWImqjHvKE254CjYUyLB)ceQ18mWktEEwP5LP5OGAxHUKQKUeNyuRPJd3AzCfe0IZ6cEJrx4luR5baGrPm55zLMaaOTCS5(XHpZp5sSS3mA8axWBm6cFw5q7OkAcaaTfUnxISlw52VmFLdTJQOPltZrb1UcDjvjDjoXOwthhU1Y4kiOfN19Jd34NCjw2BgnEGl4ngDrCfct7OkAcaaTfUnxISlw52VODfct7OkA6Y0CuqTRqxsvsxItmQ10XHBTmUccAXzD)4Wd(jxIL9MrJh4IaHkU(fVzu6GkyriiEnYKaPS3mAWpwtmTJuyGqarik9zNZO1Li7IvU9lSYKNNvAAyRy(LP5OGAxHUKQKUeNyuRPJd3AzCfe0IZ6cEJrxIszYZZknbaWakMF)4WNHp5sSS3mA8axeiuX1VSbggcYt5g415aei3lr2fRC7xynSAdSwZ7Y0CuqTRqxsvsxItmQ10XHBTmUccAXzDbVXOlrPHvdaGrP18UFC4WMp5sSS3mA8axWBm6sSg58OeaafUv00Li7IvU9luJCEuAqZTIMUmnhfu7k0LuL0L4eJAnDC4wlJRGGwCw3pohRp5sSS3mA8axeiuX1V0qEblYaLewr6zxWeM26nJshec1AEvAAq7kegiL9Mrd(XAIPDKcdecicrPp7CAY9kn1GqOwZZaRm55zLMgWAIPDKcZgycZqc1mdAUfQCqeH1vQNDdmmeSzC51gMxXcbrewxP(sKDXk3(fiuR5zGvM88SsZltZrb1UcDjvjDjoXOwthhU1Y4kiOfN1f8gJUWxOwZdaaJszYZZknbaqBHDBUFCoRp5sSS3mA8axeiuX1V4nJshKxxmTRqyGu2Bgn4FdmmemeP0(g15aeryDLAya7GZWLi7IvU9lHiL23OohxMMJcQDf6sQs6sCIrTMooCRLXvqqloRl4ngDjQiL23OohaaOTlBUFCoW)KlXYEZOXdCrGqfx)YgyyiyiRXixLMajqeH1vQHbSdodxISlw52VeYAmYvPjq6Y0CuqTRqxsvsxItmQ10XHBTmUccAXzDbVXOlrL1yKRstGeaaA7YM7hNdNp5sSS3mA8axWBm6IamheQsZlr2fRC7x0aZbHQ08Y0CuqTRqxsvsxeiuX1VG1et7ifgieqeIsF250K7vAQb1aZbHQ00awtmTJuyxItmQ10XHBTmUccAXzD)4CG9p5sSS3mA8axeiuX1VG1et7ifgieqeIsF250K7vAQbznSAdSwZZawtmTJuyxISlw52VWAy1gyTM3LP5OGAxHUKQKUeNyuRPJd3AzCfe0IZ6cEJrxIsdRgaaJsR5baG2US5(X5mZp5sSS3mA8axWBm6sCfu07Yj0Li7IvU9lifu07Yj0LP5OGAxHUKQKUeNyuRPJd3AzCfe0IZ6(X5y8tUel7nJgpWf8gJUevwJrUknbsaaOTWT5sKDXk3(LqwJrUknbsxMMJcQDf6sQs6sCIrTMooCRLXvqqloR7hNtWp5sSS3mA8axWBm6safq2GqHL2baqBHBZLi7IvU9lBfq2GqHL2Vmnhfu7k0LuL0L4eJAnDC4wlJRGGwCw3VFr4sqvZk(U9sLhhUro3)ba]] )

    storeDefault( 'SimC Import: precombat', 'actionLists', 20161016.1, [[d4cCbaGEqu7cs2gKAMuvz2iUnQStfTxYULA)iPHHu)wPHcImyuLHtLoiO6yICorfleIwkQQftvwouEOOQEQQhdvRdemrqOPIetwHPl5IuHlt56IQSzQQ6ZO47GIdd8CuA0qyzIYjPQCprLoTW5bj)fK6zurFduALefDhnWJydHuFc4m9hC5tLNpo3fRGavEUyg(Y5bkDiA(dYJucPoFJyawtZm6e6enDoOs6hhlClDD44vSnROOzsu0D0apInes9JJfULETmmedf(UKXctZQdhJHvVbCwUlzanmamD4EbjkO0D3k2w3xpcCqTy692MoFJyawtZm6e6eSOODM0NaothsBfBRsZmrr3rd8i2qi1Naot3VGbr1rZqL3regzOd3lirbLojyquD0mqZIimYq3xpcCqTy692MoFJyawtZm6e6eSOODMuPs)UgEaibKbvSTMzODQsc]] )

    storeDefault( 'Wordup\'s Wowhead APL', 'actionLists', 20161016.1, [[dWJznaGEqk0MivXUa12iQ2hbvZeK0SrmFqk1NivfDBsCzu7eWEf7wQ9RumkLsnmI8BkESQAOGuWGPQA4e4GKsNcK4yu6CGu1cjulLKAXQYYr6HGu5PqldipxjteKIMkPYKPY0vCrsXHL8mc56a1gjvPZtvAZkvBNGYPv5RkL4ZG47uv03ijttPKgnr51uv6KufZJuvDnQkCpsvH)sqghPQ0HaPKJn6cQPRhHDrCq8tpbtWGqtEVatM8cQMjCT4aasYk3kjjc2gefW)Rih0ynNPdai5IcQ9pNPxrxaSrxqnD9iSlIdcukCqXgWeht3V1eunt4AXbaKKvUvfSKiBqpT7(1yObBtZb1(oYnEd(mGjoMUFRji(PNGj4ueUhys1Uf5Cmm31JWo98aVVdtQ2TiNJHPSsD9s)q(Umbau0futxpc7I4G4NEcMGtr4EGjv7wKZXWCxpc70Zd8(omPA3ICogMYk11l9d57cQMjCT4aasYk3QcwsKnON2D)Am0GTP5GaLchuT5777gMgu77i34ni1899DdtZearrxqnD9iSlIdIF6jycofH7bwwDK1yOkWCxpc70Zd8(o8o1SMhTAhmLvQRx6hY3funt4AXbaKKvUvfSKiBqpT7(1yObBtZbbkfoOEPM18Ov7YeGTgDb101JWUioiqPWb1lPu4X0qaZbvZeUwCaajzLBvbljYg0t7UFngAW20Cq8tpbtWh49D4DsPWJPHaMHPSsD9s)q(UmbWhrxqnD9iSlIdIF6jycgunt4AXbaKKvUvfSKiBqpT7(1yObBtZbbkfo4wU2TmnKGAFh5gVb951ULPHKjaYJUGA66ryxehe)0tWemOAMW1IdaijRCRkyjr2GEA39RXqd2MMdcukCqTRp3UQ)CqTVJCJ3G16ZTR6pNjaQIUGA66ryxehe)0tWe8bEFhENuk8yAiGzyWc0Z2qRPiCpWYQJSgdvbM76ryh0gA)aVVdVtnR5rR2bdwaucQMjCT4aasYk3QcwsKnON2D)Am0GTP5GaLcheAAmkB8VLRDRGAFh5gVbDgJIq(8A3kta03OlOMUEe2fXbXp9embdQMjCT4aasYk3QcwsKnON2D)Am0GTP5GaLchuVmTiB8Jco6nb1(oYnEdUZ0Ii0sWrVjtaG(OlOMUEe2fXbXp9embHwZ999AibvZeUwCaajzLBvbljYg0t7UFngAW20CqGsHdQxWuVB8B234x7rdQ9DKB8gChm1RqMDHQJMjawPOlOMUEe2fXbXp9embR)CcJfIBw54LW1hIcQ9DKB8g8tRLmHiheztFnKGEA39RXqd2MMdQMjCT4aasYk3QcwsKniqPWbHoATKTXpupiYM(AiB8VTwJgOKjawB0futxpc7I4GaLchehdvzn0ZxoOAMW1IdaijRCRkyjr2GEA39RXqd2MMdQ9DKB8gCngQYAONVCq8tpbtWmbWck6cQPRhHDrCqGsHdULRDRHE(YbvZeUwCaajzLBvbljYg0t7UFngAW20CqTVJCJ3G(8A3AONVCq8tpbtWmbWkk6cQPRhHDrCq8tpbtW6pNWyH4MvoEjC9bOGAFh5gVb)0AjtiYbr20xdjON2D)Am0GTP5GQzcxloaGKSYTQGLezdcukCqOJwlzB8d1dISPVgYg)BdDeUegdLmbWU1OlOMUEe2fXbXp9emb)gdXz8zd)mGjoMUFRb(lROq4Lq706pNPlIWTWQ8rq1mHRfhaqsw5wvWsISb90U7xJHgSnnheOu4GInGjoMUFRzJ)TTqjO23rUXBWNbmXX09BnzcG1hrxqnD9iSlIdIF6jycofH7bwwDK1yOkWCxpc70Zd8(o8o1SMhTAhmLvQRx6FRW(iOAMW1IdaijRCRkyjr2GEA39RXqd2MMdcukCq9snR5rR2TX)2wOeu77i34n4o1SMhTAxMayLhDb101JWUioi(PNGj4d8(o8oPu4X0qaZWuwPUEP)Tc7JGQzcxloaGKSYTQGLezd6PD3VgdnyBAoiqPWb1lPu4X0qaZB8VTfkb1(oYnEdUtkfEmneWCMayvfDb101JWUioi(PNGj4ueUhyJWy6xwrHWWCxpc70JsXK1qnkWFWuk3JWHEPGQzcxloaGKSYTQGLezd6PD3VgdnyBAoiqPWbH6br20xdzJFXgYeu77i34ni5GiB6RHi0ZqMmbWQVrxqnD9iSlIdIF6jyc(aVVdlZmcjRAhmybbvZeUwCaajzLBvbljYg0t7UFngAW20CqGsHdc1sy1g)qTwYcQ9DKB8gKucReIulzzcGf6JUGA66ryxehe)0tWemOAMW1IdaijRCRkyjr2GEA39RXqd2MMdcukCqnfDKX9g)OGZxoO23rUXBqUOJmUfAj48LZeaqsrxqnD9iSlIdIF6jycofH7b(tRLSRHi0AmufyURhHD6rPyYAOgf4pykL7r46Ruq1mHRfhaqsw5wvWsISb90U7xJHgSnnheOu4GqhTwY24hQheztFnKGAFh5gVb)0AjtiYbr20xdjtaazJUGA66ryxehe)0tWeuPyYAOgf4pykL7r4wRuq1mHRfhaqsw5wvWsISb90U7xJHgSnnheOu4Gi42X0RHeu77i34n4cC7y61qYeaqGIUGA66ryxehe)0tWeuPyYAOgf4pykL7r4qVuq1mHRfhaqsw5wvWsISb90U7xJHgSnnheOu4GqTewTXpuRLSn(32cLGAFh5gVbjLWkHi1swMaasu0futxpc7I4G4NEcMGbvZeUwCaajzLBvbljYg0t7UFngAW20CqGsHdk2aM4y6(TMn(3geucQ9DKB8g8zatCmD)wtMaaARrxqnD9iSlIdcukCq1MVVVBy6g)BBHsq1mHRfhaqsw5wvWsISb90U7xJHgSnnhu77i34ni1899DdtdIF6jycMjaG8r0futxpc7I4G4NEcMGtr4EGFgWeht3V1aZD9iSlOAMW1IdaijRCRkyjr2GEA39RXqd2MMdcukCq9skfEmneW8g)Bdckb1(oYnEdUtkfEmneWCMmbbkfo4wmuhyb6Z1g)BX4tzC524xl0auZKa]] )


    storeDefault( 'Enhancement Primary', 'displays', 20161009.1, [[dKZKeaGEKk7ceABIIMPGkZwKBQI62KCyvTtI2R0Uj1(P6NGOgMa)wHNPImuKmykdNGdkuhfPQJb0JvHfckwkOQflKLROhQs9uultqEoenrqLPQsMmO00rCrr1vfu4YkDDaBeP05fL2mi12HuFuqvNgQPbI8DqGrkOuRtuWOHW4rkoji5we6Ack5EGG(mK8nrHETGIUG9Qm9alWcRB0o0e3ymDBLGHktHMYnr3U(jQL0OYutS6NzD7(fi4H2TyG5xMjJPkAI1O2z55MkFNlVYHVCyGCDJf2uI20JerJktHo3nr3U(jQL0OYuOZDt0n4wOFGePWuMcDUBIUDpurpPrLp)0Gvak3UWQTYtbLPhybwK9QsWEvox)rPf2ct54dcEODlCyKKkdvw(QTmJv3UbLsymjzWnH5EmurpPm8BAFKBLHcaZemJqm4eyz(yIfiLjy1cHbLuzOEvox)rPf2ct54dcEODlCyKKkbllF1wMXQB3GsjmMKm4gSl0pqIug(nTpYTYqbGzcMrigCcSKsktHMYnr3GBH(bse3Itci(kbltHMYnr3UhQON0OYuOPCt0n4wOFGePWuoBLIHofuogY5Uj625NgScqvzqz430(i3kdfaMjyqWjicwMEGfyDloHrPvRMu(Omf6C3eD7EOIEIBXjbeFLGLPqt5MOBx)e1sClojG4ReSmfAk3eD7EOIEIBXjbeFLGLPPYGYXhe8q729lqWdnYctzw4pWAuvgwLzHnLOn9ir429inM9Q8xjy5OkblJQsWYZkblPmlSh4pHP7j4HUYqzEQ89qiRBxJY5AeV(yvRM4wmKZltpWcSUbhEUhe8qxgEOcFyFvMAIv)mRB0o0e3ymDBLGHktpWcSW6guhdTBmMUTsifuw(QTCUgXRpw1QjUrnXQFMTCU(JslSfMY0o0KYXt8NCt(Z5acktnXQFM1nOogA3ymDBLqkOCmKZDt0TZynwbOQ8uz4FnQ1TBe7ryI1Ok)r4eMKTmuhdns3yediqxjKkF9PvtCl8ZbGqLbLzSgvADt0TZpnyfGQsWYutS6NzD7(fi4HUmY5tWJYuOZDt0TRFIAjUfNeq8vcwogGmCt0TZynwbOQmOmCl0pqI0OYXaKHBIUD(PbRauvguMXAuP1nr3oJ1yfGQYGYuOZDt0n4wOFGeXT4KaIVsWY8Xelqkxsl]] )

    storeDefault( 'Enhancement AOE', 'displays', 20161009.1, [[dKdHeaGEKQ2fqPTjsAMafMTq3ufCBbhwv7KO9kTBsTFQ(Pivdtu(TIEgi1qrYGPmCcoijDucPJbPhRGfcswkiAXIQLd4Ha5POwgO8CKYebHPQIMScz6qDrrCvcvCzLUUkTrcLZRcTzfQTdHpks40iMgsfFxKOrsOsRJqvJgunEcXjbQUfjUMiL7bu0NHOXHujVgPsDr7zzrV7Dh5Mytn2nMq)wjkSYuiOCtXTZha5IBEzkas4bo6gOxatMA3uVaF5YaBSmOe5zcKLfhARBSWgJIfFAWBEzkejUP425dGCXnVmfIe3uCdID8FJ4cvzkejUP4gOzi)XnV8Hxes4gC7Ke2kHoRSO39U06zLO9SCI(ZJ7Ocvz1bmzQDdmi0WvcRS8dBzMea5g4bHjaw8UjaSdZq(Jld5g3N2wjSm00YsfSWGHwMhaic4YysybZSIRewplNO)84oQqvwDatMA3adcnCLOLLFylZKai3apimbWI3Tr74)gXLHCJ7tBRewgAAzPcwyWqlU4YuiOCtXni2X)nIDtnka)ReTmfck3uCd0mK)4MxMcbLBkUbXo(VrCHQ8XkvqPR0kRMEIBkUD4fHeUHkZkd5g3N2wjSm0urZYGgSOLf9U31n1ibPoSAC5HYuisCtXnqZq(JDtnka)ReTmfck3uC78bqUy3uJcW)krltHGYnf3and5p2n1Oa8Vs0Y8aaraxUmfIe3uCdID8FJy3uJcW)krlZenY46MIBhiAs4gQe6YSWgJIfFAWDd0mob6z5Vs0YavIwgzLOLZReT4YGMchD7Cword)1dBy1y3utpPS6fpDtXTdViKWnuzwzrV7DDdccWoGjtDzibpfI7zzi2X)nIBEzrV7Dh5g4dtTBmH(Ts6Kvw9INUP42bIMeUHkHUCI(ZJ7OcvzkejUP425dGCXUPgfG)vIwMcGeEGJUb6fWKPU8OD8FJ4YQPN4MIBhiAs4gQe6YmrJmUUP42Hxes4gQmR85hxn2TuamVcvMvg8HPMMBm8zk1vsNYq(AKRBGGVd0nrJS8NtIe8XYSWoq(iH(htM6kHLk0LfBQXLvbiF0n5damtzz5h2YjA4VEydRg7MA6jLPaiHh4OBGpm1UXe63kPtwzkas4bo6Mytn2nMq)wjkSYSWpq0iRmTYQdyYu7gOxatMAAfQYIuzwXT]] )


end